// FormPostSerializer.h : Declaration of the CFormPostSerializer

#ifndef __FORMPOSTSERIALIZER_H_
#define __FORMPOSTSERIALIZER_H_

#include "resource.h"       // main symbols

// includes from BizTalk SDK from "Program Files\Microsoft BizTalk Server\SDK\Include"
#include <BTSSerializerComps.h> // for IBizTalkSerializerComponent interface
#include <bts_sdk_guids.h>  // for guids for interfaces and categories

// includes from STL
#include <vector>

#ifndef _CPPUNWIND
#pragma message("***************************************************************************************************")
#pragma message("*** Exception handling must be enabled for the STL classes used here.")
#pragma message("***************************************************************************************************")
#endif



/////////////////////////////////////////////////////////////////////////////
// CFormPostSerializer
class ATL_NO_VTABLE CFormPostSerializer : 
    public CComObjectRootEx<CComMultiThreadModel>,
    public CComCoClass<CFormPostSerializer, &CLSID_FormPostSerializer>,
    public IDispatchImpl<IFormPostSerializer, &IID_IFormPostSerializer, &LIBID_FORMPOSTBATCHSERIALIZERLib>,
    public IBizTalkSerializerComponent
{
public:
    CFormPostSerializer()
    :   m_nDocHandle(0), m_nDocSize(0)
    {
    }
    ~CFormPostSerializer()
    {
    }

DECLARE_REGISTRY_RESOURCEID(IDR_FORMPOSTSERIALIZER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

// BizTalk requires a COM category for custom serializers
//         and a COM category for each BizTalk component
BEGIN_CATEGORY_MAP(CFormPostSerializer)
    IMPLEMENTED_CATEGORY(CATID_BIZTALK_COMPONENT)
    IMPLEMENTED_CATEGORY(CATID_BIZTALK_SERIALIZER)
END_CATEGORY_MAP()

BEGIN_COM_MAP(CFormPostSerializer)
    COM_INTERFACE_ENTRY(IBizTalkSerializerComponent)
    COM_INTERFACE_ENTRY(IFormPostSerializer)
END_COM_MAP()

// IBizTalkSerializerComponent interface
public:
    STDMETHOD(Init)(BSTR bstrSourceQualifer, BSTR bstrSourceID, BSTR bstrDestinationQualifier, BSTR bstrDestinationID,
                    long nEnvelopeID, IDictionary * pdictDelimiters, IStream * pstmOutput, long nNumDocs, long nPortID);
    STDMETHOD(AddDocument)(long nDocHandle, IDictionary * pdictTransport, BSTR bstrTrackID, long nChannelID);
    STDMETHOD(GetInterchangeInfo)(BSTR * pbstrInterchangeID, long * pnNumGroups);
    STDMETHOD(GetGroupInfo)(long * pnNumDocs, LARGE_INTEGER * pnGroupStartOffset, long * pnGroupLen);
    STDMETHOD(GetDocInfo)(long * pnDocHandle, BOOL * pbSizeFromXMLDoc, LARGE_INTEGER * pnDocStartOffset, long * pnDocLen);

private:
    CComPtr<IStream> m_spstmOutput; // holds stream for serialized data; initialized in Init()

    long m_nDocHandle, m_nDocSize; // stores the handle and size for use by GetDocInfo(); initialized in AddDocument()
        // Because BizTalk 2002 is limited to only one document in the interchange, we
        // do not need to store a list of handles and sizes.  Single data members are enough.

// IFormPostSerializer
public:
};

#endif //__FORMPOSTSERIALIZER_H_
